
`default_nettype none

module top(
	input  wire Reset,				// Pin 17
	
	input wire Encoder1A_i,			// Pin 68
	input wire Encoder1B_i,			// Pin 67
	input wire Encoder1S_i,			// Pin 66
	input wire Encoder2A_i,			// Pin 71
	input wire Encoder2B_i,			// Pin 70	
	output wire [7:0] Cathodes_o,
	output wire [7:0] Segments_o
);
	
	// Generator sygnau zegarowego
	parameter CLOCK_HZ = 14_000_000;
	wire Clock;
	OSCH #(
		.NOM_FREQ("14.00")
	) OSCH_inst(
		.STDBY(1'b0),
		.OSC(Clock),
		.SEDSTDBY()
	);
	
	// Encoder 1
	wire AddressIncrement;
	wire AddressDecrement;
	wire WriteExecute;
	
	Encoder Encoder1(
		.Clock(Clock),
		.Reset(Reset),
		.AsyncA_i(Encoder1A_i),
		.AsyncB_i(Encoder1B_i),
		.AsyncS_i(Encoder1S_i),
		.Increment_o(AddressIncrement),
		.Decrement_o(AddressDecrement),
		.ButtonPress_o(WriteExecute),
		.ButtonRelease_o(),
		.ButtonState_o()
	);
	
	// Encoder 2
	wire DataIncrement;
	wire DaraDecrement;
	
	Encoder Encoder2(
		.Clock(Clock),
		.Reset(Reset),
		.AsyncA_i(Encoder2A_i),
		.AsyncB_i(Encoder2B_i),
		.AsyncS_i(1'b0),
		.Increment_o(DataIncrement),
		.Decrement_o(DaraDecrement),
		.ButtonPress_o(),
		.ButtonRelease_o(),
		.ButtonState_o()
	);
	
	// Address pointer
	reg [7:0] Address;
	
	always @(posedge Clock, negedge Reset) begin
		if(!Reset)
			Address <= 0;
		else if(AddressIncrement)
			Address <= Address + 1'b1;
		else if(AddressDecrement)
			Address <= Address - 1'b1;
	end
	
	// Data to write
	reg [7:0] DataToWrite;
	
	always @(posedge Clock, negedge Reset) begin
		if(!Reset)
			DataToWrite <= 0;
		else if(DataIncrement)
			DataToWrite <= DataToWrite + 1'b1;
		else if(DaraDecrement)
			DataToWrite <= DataToWrite - 1'b1;
	end
	
	// RAM instance
	wire [7:0] DataReadFromMemory;
	
	RAM #(
		.ADDRESS_WIDTH(8),
		.DATA_WIDTH(8)
	) RAM_inst(
		.Clock(Clock),
		.Reset(Reset),
		.ReadEnable_i(1'b1),
		.WriteEnable_i(WriteExecute),
		.Address_i(Address),
		.Data_i(DataToWrite),
		.Data_o(DataReadFromMemory)
	);
	
	// Display instance
	DisplayMultiplex #(
		.CLOCK_HZ(CLOCK_HZ),
		.SWITCH_PERIOD_US(1000),
		.DIGITS(8)
	) DisplayMultiplex_inst(
		.Clock(Clock),
		.Reset(Reset),
		.Data_i({8'd0, Address, DataToWrite, DataReadFromMemory}),
		.DecimalPoints_i(8'b00010100),
		.Cathodes_o(Cathodes_o),
		.Segments_o(Segments_o),
		.SwitchCathode_o()
	);
	

endmodule

`default_nettype wire
